/*
 * FFT Class
 */

import ddf.minim.analysis.*;
import ddf.minim.*;


class AnalyseFFT {
  PApplet theApplet;
  Minim minim;
  AudioInput in;
  FFT fft;
  int numBands;
  boolean isLoop;



  AnalyseFFT(PApplet _applet) {
    this.theApplet = _applet;
    minim = new Minim(theApplet);
    in = minim.getLineIn();  
  }


  /**
   *
   * Configure settings
   */

  void setSettings(int _bands) {
    int bufferSize = this.in.mix.size();
    println("AUDIO SETTINGS:");
    println("> Buffer size = "+bufferSize);
    // a bufferSize of 512 results in the sharpest, most responsive movement
    // 512 returns 86hz bandwidth which allows 9 octaves / bands
    // 1024 returns 43hz bandwidth which allows 10 octaves / bands
    float sampleRate = this.in.sampleRate();
    println("> Sample rate = "+sampleRate);

    //Initialise FFT
    fft = new FFT(bufferSize, sampleRate);

    // size of the smallest octave to use (in Hz) so we calculate averages based on a miminum octave width of 86 Hz
    int fft_base_freq = 43; 
    // how many bands to split each octave into? in this case split each octave into 1 band
    int  fft_band_per_oct = _bands; 
    fft.logAverages(fft_base_freq, fft_band_per_oct); // results in 9 bands
    fft.window(FFT.HAMMING); // type of FFT analysis
    numBands = fft.avgSize(); // avgSize() returns the number of averages currently being calculated
    println("> Number of bands: " + numBands); // returns 10 bands
    println("//////////////////////");
  }
  
  /**
   * returns num of analysed bands
   */
  
  int getNumBands(){
   return numBands; 
  }
  
  void beginAnalyse(){
   if( isLoop ){
//     thePlayer.loop();
   }else {
//     thePlayer.play();
   }   
   fft.forward(in.mix);
  }


  float analyseBand(int i, boolean showInfo) {
    float avAmp = fft.getAvg(i); // return the value of the requested average band, ie. returns averages[i]
    //println("Averages " + i + " : " + avAmp);

    // calculating average for each band will enable us to determine
    // the avergage amplitude for a set frequency (I think !!)  
    float centerFrequency = fft.getAverageCenterFrequency(i);

    // how wide is this average in Hz?
    float averageWidth = fft.getAverageBandWidth(i);       

    // we calculate the lowest and highest frequencies
    // contained in this average using the center frequency
    // and bandwidth of this average.
    float lowFreq  = centerFrequency - averageWidth/2;
    float highFreq = centerFrequency + averageWidth/2;

    if (showInfo) {
      println("                                              ");
      println("SHOWING INFO FOR BAND : "+i+" //////////////////////////////////////");  
      println("Average signal 'ENERGY' = "+ avAmp);
      println("Frequency average for band "+i+" = "+centerFrequency);
      println("Average Frequency band width = "+averageWidth);
      println("Lowest freq = "+lowFreq);
      println("Highest freq = "+highFreq);
      println("FIN______/////////////////////////////////////");
      println("                                              ");
    }

    return avAmp;
  }
}
