// P_3_1_1_01.pde
//
// Generative Gestaltung, ISBN: 978-3-87439-759-9
// First Edition, Hermann Schmidt, Mainz, 2009
// Hartmut Bohnacker, Benedikt Gross, Julia Laub, Claudius Lazzeroni
// Copyright 2009 Hartmut Bohnacker, Benedikt Gross, Julia Laub, Claudius Lazzeroni
//
// http://www.generative-gestaltung.de
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * typewriter. time reactive.
 *
 * MOUSE
 * position y           : adjust spacing (line height)
 *
 * KEYS
 * a-z                  : text input (keyboard)
 * backspace            : delete last typed letter
 * ctrl                 : save png + pdf
 */

import processing.pdf.*;
import java.util.Calendar;

boolean doSave = false;

String textTyped = "";
float[] fontSizes = new float[textTyped.length()];
float[] fontColours = new float[textTyped.length()];
float[] fontNoir = new float[textTyped.length()];
float minFontSize = 30;
float maxFontSize = 400;
float newFontSize = 0;

int pMillis;
float maxTimeDelta = 3000.0;

float spacing = 70; // line height
float tracking = 0; // between letters
PFont font;
float teinte;
float noir;
float d;
////////////// SON
AnalyseFFT myFFT;
float ampBand_02;

void setup() {
  //size(1100, 1555);
  fullScreen();
  background(0, 0, 100);
  colorMode(HSB, 360, 100, 100, 100);
  myFFT = new AnalyseFFT(this);
  myFFT.setSettings(1);

//Lao MN
//Impact
  font = createFont("Lao MN", 15);
  smooth();
  
  // init fontSizes
  for (int i = 0; i < textTyped.length(); i++) {
    fontSizes[i] = minFontSize;
  }

  pMillis = millis();

}


void draw() {
  background(0, 0, 100);



  //////////////////////// SON
  myFFT.beginAnalyse();
  // on parcourt les différentes bandes d'audio
  for (int i= 0; i<myFFT.getNumBands(); i++) {
    ampBand_02 = myFFT.analyseBand(2, true); // bande 3 ou 4
  }

  //MAP teinte
  teinte = map(ampBand_02, 0, 5, 0, 340);

  /////////////////////// TEXTE
  textAlign(LEFT);
  noStroke();
  
  if (mousePressed) {
      spacing = map(mouseY, 0,height, 0,120);
  }

  translate(10, 220+spacing);

  float x = 0, y = 0, fontSize = 20;
  d = myFFT.in.mix.level()*10000;


  for (int i = 0; i < textTyped.length(); i++) {
    // get fontsize for the actual letter from the array
    fontSize = fontSizes[i];
    teinte = fontColours[i];
    noir = fontNoir[i];
    
     fill(teinte, 95, noir, 80);
     textFont(font, fontSize);
   
    char letter = textTyped.charAt(i);
    float letterWidth = textWidth(letter) + tracking;



    if (x+letterWidth > width-20) {
      // start new line and add line height
      x = 0;
      y += spacing;
    }

    // draw letter
    text(letter, x, y);
    // update x-coordinate for next letter
    x += letterWidth;
  }

  if (doSave) {
    doSave = false;
    saveFrame(timestamp()+"_##.png");
  }


  // blinking cursor after text
  float timeDelta = millis() - pMillis;
  newFontSize = map(timeDelta, 0, maxTimeDelta, minFontSize, maxFontSize);
  newFontSize = min(newFontSize, maxFontSize);

  fill(186, 97, 60);
  if (frameCount/20 % 2 == 0) fill(0, 0, 100);
  rect(x, y, newFontSize/2, newFontSize/20);
  
     if (keyCode == LEFT)font = createFont("Lao MN", 15);
    if (keyCode == RIGHT)font = createFont("Impact", 15);
    if (keyCode == UP)font = createFont("Times New Roman", 15);
    if (keyCode == DOWN)font = createFont("Comic Sans MS", 15);

}



void keyReleased() {
  // export pdf and png
  if (keyCode == CONTROL) doSave = true;
}

  
void keyPressed() {

  if (key != CODED) {
    switch(key) {
    case DELETE:
    case BACKSPACE:
      if (textTyped.length() > 0) {
        textTyped = textTyped.substring(0, max(0, textTyped.length()-1));
        fontSizes = shorten(fontSizes);
      }
      break;
      // disable those keys
    case TAB:
    case RETURN:
    case ESC:
      break;
    default:
      textTyped += key;
      fontSizes = append(fontSizes, newFontSize);
      fontColours = append(fontColours, ampBand_02*100); // multiplicateur
      fontNoir = append(fontNoir,d); // multiplicateur
    }

    // reset timer
    pMillis = millis();
  }
}

// timestamp
String timestamp() {
  Calendar now = Calendar.getInstance();
  return String.format("%1$ty%1$tm%1$td_%1$tH%1$tM%1$tS", now);
}
